@@ -684,6 +684,9 @@ TkpConfigureMenuEntry(
 		  NSArray *itemArray = [submenu itemArray];
 		  for (NSMenuItem *item in itemArray) {
 		    TkMenuEntry *submePtr = menuRefPtr->menuPtr->entries[i];
+		    /* Work around an apparent bug where itemArray can have
+                      more items than the menu's entries[] array. */
+                    if (i >= menuRefPtr->menuPtr->numEntries) break;
 		    [item setEnabled: !(submePtr->state == ENTRY_DISABLED)];
 		    i++;
 		  }
@@ -754,10 +757,18 @@ TkpPostMenu(
 				 * to be posted. */
     int y)			/* The global y-coordinate */
 {
-    NSWindow *win = [NSApp keyWindow];
-    if (!win) {
+   
+
+    /* Get the object that holds this Tk Window.*/
+    Tk_Window root;
+    root = Tk_MainWindow(interp);
+    if (root == NULL) {
 	return TCL_ERROR;
     }
+ 
+    Drawable d = Tk_WindowId(root);
+    NSView *rootview = TkMacOSXGetRootControl(d);
+    NSWindow *win = [rootview window];
 
     inPostMenu = 1;
 
